import {TagAccessor} from './tag';
import {check_type} from '../util/type_check'

export interface DocumentBody{
    title           : string,
    content_type    : string,
    basepath        : string,
    filename        : string,
    content_hash    : string,
    additional      : JSONMap,
    tags            : string[],//eager loading 
}

export const MetaContentBody = {
    title         : "string",
    content_type  : "string",
    basepath      : "string",
    filename      : "string",
    content_hash  : "string",
    additional    : "object",
    tags          : "string[]",
}

export const isDocBody = (c : any):c is DocumentBody =>{
    return check_type<DocumentBody>(c,MetaContentBody);
}

export interface Document extends DocumentBody{
    readonly id: number;
    readonly created_at:number;
    readonly deleted_at:number|null;
};

export const isDoc = (c: any):c is Document =>{
    if('id' in c && typeof c['id'] === "number"){
        const {id, ...rest} = c;
        return isDocBody(rest);
    }
    return false;
}

export type QueryListOption = {
    /**
     * search word
     */
    word?:string,
    allow_tag?:string[],
    /**
     * limit of list
     * @default 20
     */
    limit?:number,
    /**
     * use offset if true, otherwise
     * @default false
     */
    use_offset?:boolean,
    /**
     * cursor of documents
     */
    cursor?:number,
    /**
     * offset of documents
     */
    offset?:number,
    /**
     * tag eager loading
     * @default true
     */
    eager_loading?:boolean,
    /**
     * content type
     */
    content_type?:string
}

export interface DocumentAccessor{
    /**
     * find list by option
     * @returns documents list
     */
    findList: (option?:QueryListOption)=>Promise<Document[]>,
    /**
     * @returns document if exist, otherwise undefined
     */
    findById: (id:number,tagload?:boolean)=> Promise<Document| undefined>,
    /**
     * find by base path and filename.
     * if you call this function with filename, its return array length is 0 or 1.
     */
    findByPath:(basepath: string,filename?:string)=>Promise<Document[]>;
    /**
     * find deleted content
     */
    findDeleted:(content_type:string)=>Promise<Document[]>;
    /**
     * update document except tag.
     */
    update:(c:Partial<Document> & { id:number })=>Promise<boolean>;
    /**
     * add document
     */
    add:(c:DocumentBody)=>Promise<number>;
    /**
     * delete document
     * @returns if it exists, return true.
     */
    del:(id:number)=>Promise<boolean>;
    /**
     * @param c Valid Document
     * @param tagname tag name to add
     * @returns if success, return true
     */
    addTag:(c:Document,tag_name:string)=>Promise<boolean>;
    /**
     * @returns if success, return true
     */
    delTag:(c:Document,tag_name:string)=>Promise<boolean>;
};